const HitBTC = require('hitbtc-api').default;
const _ = require('lodash');
const moment = require('moment');

const errors = require('../exchangeErrors');
const retry = require('../exchangeUtils').retry;

const Trader = function(config) {
  this.api = new HitBTC({
    key: config.key,
    secret: config.secret,
    isDemo: false
  });


  this.currency = config.currency;
  if(this.currency === 'USDT') {
    // what they call USDT is referred
    // to as USD in their API.
    this.currency = 'USD';
  }

  this.asset = config.asset;

  this.pair = this.asset + this.currency;
};

const recoverableErrors = [
  // 'SOCKETTIMEDOUT',
  // 'TIMEDOUT',
  // 'CONNRESET',
  // 'CONNREFUSED',
  // 'NOTFOUND',
  // 'Rate limit exceeded',
  // 'Response code 5',
  // 'GDAX is currently under maintenance.',
  // 'HTTP 408 Error',
  // 'HTTP 504 Error',
  // 'HTTP 503 Error',
  // 'socket hang up',
  // 'EHOSTUNREACH'
];

const includes = (str, list) => {
  if(!_.isString(str))
    return false;

  return _.some(list, item => str.includes(item));
}

Trader.prototype.processResponse = function(method, args, next) {
  const catcher = err => {

    // check for retryable errors
    if(includes(err.message, recoverableErrors)) {
      err.nonFatal = true;
    }

    return next(err);
  }

  return {
    failure: catcher,
    success: data => {
      // if(!data) {
      //   return catcher();
      // }

      // if(data.error) {
      //   return catcher(data.error);
      // }

      // if(includes(data, ['Please complete the security check to proceed.'])) {
      //   return next(new Error(
      //     'Your IP has been flagged by CloudFlare. ' +
      //     'As such Gekko Broker cannot access Coinfalcon.'
      //   ));
      // }

      next(undefined, data);
    }
  }
}

Trader.prototype.getTicker = function(callback) {
  const handle = this.processResponse(this.getTicker, [callback], (err, data) => {
    if(err) {
      return callback(err);
    }

    callback(null, {
      bid: +data.bid,
      ask: +data.ask
    });
  });

  const fetch = cb => this.api.getTicker(this.pair).then(handle.success).catch(handle.failure);
  retry(null, fetch, handle);
};

Trader.getCapabilities = function() {
  return {
    name: 'HitBTC',
    slug: 'hitbtc',
    currencies: ['USD'],
    assets: ['BTC'],
    markets: [
      { pair: ['USD', 'BTC'], minimalOrder: {
        amount: 0.01,
        unit: 'asset',
        price: 0.01
      } },
    ],
    requires: ['key', 'secret'],
    providesHistory: 'date',
    providesFullHistory: true,
    // tid: 'tid',
    tradable: true,
    gekkoBroker: '0.6.2'
  };
};


const a = new Trader({
  currency: 'USDT',
  asset: 'BTC'
});

module.exports = Trader;
